import 'dart:convert';

import 'package:cometchat_calls_sdk/cometchat_calls_sdk.dart';

import '../helper/api_connection.dart';
import '../helper/cometchatcalls_utils.dart';
import '../model/call_log_filter_params.dart';

/// `CallLogRequest` is a class that encapsulates a request for call logs.
///
/// It includes parameters for filtering and pagination of the call logs.
class CallLogRequest {
  final _tag = "CallLogRequest";

  /// The type of the call(s). It can be 'audio' or 'video'.
  String? callType;

  /// The status of the call(s). It can be 'initiated', 'ongoing', 'ended', etc.
  String? callStatus;

  /// The category of the call(s). It can be 'direct' or 'group'.
  String? callCategory;

  /// The direction of the call(s). It can be 'inbound' or 'outbound'.
  String? callDirection;

  /// The unique ID of the user involved in the call(s).
  String? uid;

  /// The unique ID of the group involved in the call(s).
  String? guid;

  /// The authentication token.
  String? authToken;

  /// Indicates if the call(s) has a recording.
  bool hasRecording = false;

  /// Pagination parameters.
  int totalPages = 0;
  int currentPage = 0;
  int maxLimit = 100;
  int limit = 30;
  int defaultLimit = 30;

  /// A private constructor used by `CallLogRequestBuilder` to create an instance of `CallLogRequest`.
  CallLogRequest._builder(CallLogRequestBuilder builder)
      : callType = builder.callType,
        callStatus = builder.callStatus,
        callCategory = builder.callCategory,
        callDirection = builder.callDirection,
        uid = builder.uid,
        guid = builder.guid,
        authToken = builder.authToken,
        limit = builder.limit,
        hasRecording = builder.hasRecording;

  /// Fetches the next batch of call logs.
  ///
  /// The [onSuccess] function is called with a list of [CallLog] objects when the call logs are successfully fetched.
  /// The [onError] function is called with a [CometChatCallsException] object when there is an error while fetching the call logs.
  ///
  /// Returns a [Future] that resolves to a list of [CallLog] objects.
  ///
  /// Example:
  /// ```dart
  /// fetchNext(
  ///   onSuccess: (callLogs) {
  ///     // Process the fetched call logs
  ///   },
  ///   onError: (exception) {
  ///     // Handle the error
  ///   },
  /// );
  /// ```
  Future<List<CallLog>> fetchNext(
      {required Function(List<CallLog> callLogs) onSuccess,
      required Function(CometChatCallsException excep) onError}) async {
    if (!await CometChatCalls.isInitialized()) {
      onError(CometChatCallsException(
          CometChatCallsConstants.codeCometChatCallsSDKInitError,
          CometChatCallsConstants.messageCometChatCallsSDKInit,
          CometChatCallsConstants.messageCometChatCallsSDKInit));
    } else if (authToken == null) {
      onError(CometChatCallsException(
          CometChatCallsConstants.codeUserAuthTokenNull,
          CometChatCallsConstants.messageCodeUserAuthTokenNull,
          CometChatCallsConstants.messageCodeUserAuthTokenNull));
    } else if (authToken!.isEmpty) {
      onError(CometChatCallsException(
          CometChatCallsConstants.codeUserAuthTokenNull,
          CometChatCallsConstants.messageCodeUserAuthTokenBlankOrEmpty,
          CometChatCallsConstants.messageCodeUserAuthTokenBlankOrEmpty));
    }
    if (totalPages != 0 && totalPages == currentPage) {
      onSuccess(<CallLog>[]);
      return [];
    }
    ApiConnection().getCallLogList(getParams(true), authToken!,
        (String response) {
      getCallLogList(onSuccess, onError, response);
    }, (CometChatCallsException e) {
      CometChatCallsUtils.showLog(_tag, "generateToken onError: ${e.message}");
      onError(e);
    });
    return [];
  }

  /// Fetches the previous batch of call logs.
  ///
  /// The [onSuccess] function is called with a list of [CallLog] objects when the call logs are successfully fetched.
  /// The [onError] function is called with a [CometChatCallsException] object when there is an error while fetching the call logs.
  ///
  /// Returns a [Future] that resolves to a list of [CallLog] objects.
  ///
  /// Example:
  /// ```dart
  /// fetchPrevious(
  ///   onSuccess: (callLogs) {
  ///     // Process the fetched call logs
  ///   },
  ///   onError: (exception) {
  ///     // Handle the error
  ///   },
  /// );
  /// ```
  Future<List<CallLog>> fetchPrevious(
      {required Function(List<CallLog> callLogs) onSuccess,
      required Function(CometChatCallsException excep) onError}) async {
    if (!await CometChatCalls.isInitialized()) {
      onError(CometChatCallsException(
          CometChatCallsConstants.codeCometChatCallsSDKInitError,
          CometChatCallsConstants.messageCometChatCallsSDKInit,
          CometChatCallsConstants.messageCometChatCallsSDKInit));
    } else if (authToken == null) {
      onError(CometChatCallsException(
          CometChatCallsConstants.codeUserAuthTokenNull,
          CometChatCallsConstants.messageCodeUserAuthTokenNull,
          CometChatCallsConstants.messageCodeUserAuthTokenNull));
    } else if (authToken!.isEmpty) {
      onError(CometChatCallsException(
          CometChatCallsConstants.codeUserAuthTokenNull,
          CometChatCallsConstants.messageCodeUserAuthTokenBlankOrEmpty,
          CometChatCallsConstants.messageCodeUserAuthTokenBlankOrEmpty));
    }
    if (currentPage == 1) {
      onSuccess(<CallLog>[]);
      return [];
    }
    ApiConnection().getCallLogList(getParams(false), authToken!,
        (String response) {
      getCallLogList(onSuccess, onError, response);
    }, (CometChatCallsException e) {
      CometChatCallsUtils.showLog(_tag, "generateToken onError: ${e.message}");
      onError(e);
    });
    return [];
  }

  /// Processes the response from a `getCallLogList` request.
  ///
  /// onSuccess: A function to be called when the call logs are successfully retrieved.
  /// onError: A function to be called when there is an exception.
  /// response: The raw response from the API call.
  getCallLogList(Function(List<CallLog> callLogs) onSuccess,
      Function(CometChatCallsException excep) onError, String response) {
    List<CallLog> data = <CallLog>[];
    try {
      totalPages = jsonDecode(response)['meta']['pagination']["total_pages"];
      currentPage = jsonDecode(response)['meta']['pagination']["current_page"];
      if (jsonDecode(response)["data"] != null) {
        data = <CallLog>[];
        jsonDecode(response)["data"].forEach((v) {
          data.add(CallLog.fromJson(v));
        });
        onSuccess(data);
      }
    } catch (e) {
      CometChatCallsUtils.showLog(_tag, "Error: $e");
      onError(CometChatCallsException(
          CometChatCallsConstants.codeError, e.toString(), e.toString()));
    }
  }

  /// Returns the parameters required for the `getCallLogList` function.
  ///
  /// Set `isNext` to true to load the next page, or false to load the previous page.
  CallLogFilterParams getParams(bool isNext) {
    final params = CallLogFilterParams();
    if (isNext) {
      params.page = currentPage + 1;
    } else {
      params.page = currentPage - 1;
    }
    params.perPage = limit;
    params.hasRecordings = hasRecording;
    if (callStatus != null && callStatus!.isNotEmpty) {
      params.status = callStatus;
    }
    if (callType != null && callType!.isNotEmpty) {
      params.type = callType;
    }
    if (callCategory != null && callCategory!.isNotEmpty) {
      params.mode = callCategory;
    }
    if (callDirection != null && callDirection!.isNotEmpty) {
      params.direction = callDirection;
    }
    if (uid != null && uid!.isNotEmpty) {
      params.uid = uid;
    }
    if (guid != null && guid!.isNotEmpty) {
      params.guid = guid;
    }
    return params;
  }
}

/// `CallLogRequestBuilder` is a builder class for `CallLogRequest`.
///
/// Use this class to create an instance of `CallLogRequest`.
class CallLogRequestBuilder {
  /// The type of the call(s). It can be 'audio' or 'video'.
  String? callType;

  /// The status of the call(s). It can be 'initiated', 'ongoing', 'ended', etc.
  String? callStatus;

  /// The category of the call(s). It can be 'direct' or 'group'.
  String? callCategory;

  /// The direction of the call(s). It can be 'inbound' or 'outbound'.
  String? callDirection;

  /// The unique ID of the user involved in the call(s).
  String? uid;

  /// The unique ID of the group involved in the call(s).
  String? guid;

  /// The authentication token.
  String? authToken;

  /// Indicates if the call(s) has a recording.
  bool hasRecording = false;

  /// The maximum number of call logs to return per request.
  int limit = 30;

  /// Default constructor for `CallLogRequestBuilder`.
  CallLogRequestBuilder();

  /// Builds an instance of `CallLogRequest` with the parameters set in this builder.
  CallLogRequest build() {
    return CallLogRequest._builder(this);
  }
}
